#include "vncrenderer.h"
#include <cstring>

VncRenderer::VncRenderer(QQuickWindow *window,
                         QSharedPointer<QOpenGLBuffer> vertices,
                         QSharedPointer<QVector<GLuint>> copy_textures,
                         QSharedPointer<QVector<QOpenGLTexture*>> draw_textures,
                         QObject *parent) :
    QObject(parent), m_clear(true), m_window(window), m_vertices(vertices), m_copy_textures(copy_textures), m_draw_textures(draw_textures)
{
    m_program.addShaderFromSourceFile(QOpenGLShader::Vertex, "draw_shader.vsh");
    m_program.addShaderFromSourceFile(QOpenGLShader::Fragment, "draw_shader.fsh");
    m_program.bind();
    m_program.setUniformValue("texture", 0);
    m_program.release();

    m_functions = QOpenGLContext::currentContext()->functions();
    m_functions->glActiveTexture(GL_TEXTURE0);
}

void VncRenderer::render()
{
  /*if(m_frameCount == 0)
    m_time.start();
  else
    qDebug() << (float(m_frameCount) / (float(m_time.elapsed()) / 1000.0f));
    m_frameCount++;*/
    if(m_clear) {
        m_functions->glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
        m_functions->glClear(GL_COLOR_BUFFER_BIT);
        m_clear = false;
    }

    m_program.bind();
    m_vertices->bind();
    m_program.setAttributeBuffer("position", GL_FLOAT, 0 * sizeof(float), 2, 7 * sizeof(float));
    m_program.setAttributeBuffer("color", GL_FLOAT, 2 * sizeof(float), 3, 7 * sizeof(float));
    m_program.setAttributeBuffer("texcoord", GL_FLOAT, 5 * sizeof(float), 2, 7 * sizeof(float));
    m_program.enableAttributeArray("position");
    m_program.enableAttributeArray("color");
    m_program.enableAttributeArray("texcoord");

    for(unsigned int i = 0; i < m_fill_count; i++)
        m_functions->glDrawArrays(GL_TRIANGLE_STRIP, i * 4, 4);

    for(unsigned int i = 0; i < m_copy_count; i++) {
        m_functions->glBindTexture(GL_TEXTURE_2D, m_copy_textures->at(i));
        m_functions->glDrawArrays(GL_TRIANGLE_STRIP, (m_fill_count + i) * 4, 4);
    }

    for(unsigned int i = 0; i < m_draw_count; i++) {
        m_draw_textures->at(i)->bind();
        m_functions->glDrawArrays(GL_TRIANGLE_STRIP, (m_fill_count + m_copy_count + i) * 4, 4);
        m_draw_textures->at(i)->release();
    }

    m_vertices->release();
    m_program.release();

    m_functions->glFlush();
    
    m_window->resetOpenGLState();

    emit finishedRendering();
}

QOpenGLFramebufferObject *VncRenderer::createFramebufferObject(const QSize &size)
{
    QMatrix4x4 ortho;
    ortho.ortho(QRect(QPoint(0, size.height()), QSize(size.width(), -1 * size.height())));
    m_program.bind();
    m_program.setUniformValue("ortho", ortho);
    m_program.release();

    QOpenGLFramebufferObject *framebufferObject = new QOpenGLFramebufferObject(size);
    emit FBOTextureChanged(framebufferObject->texture(), ortho);
    return framebufferObject;
}

void VncRenderer::synchronize(QQuickFramebufferObject *object)
{
    QVnc *vnc = static_cast<QVnc*>(object);
    Dispatcher *dispatcher = vnc->dispatcher();
    m_fill_count = dispatcher->fill_count();
    m_copy_count = dispatcher->copy_count();
    m_draw_count = dispatcher->draw_count();

    m_program.bind();
    m_program.setUniformValue("colormax", dispatcher->colorMax());
    m_program.release();
}
