#ifndef VNCRENDERER_H
#define VNCRENDERER_H

#include <QQuickWindow>
#include <QQuickFramebufferObject>
#include <QOpenGLFramebufferObject>
#include <QOpenGLContext>
#include <QOpenGLFunctions>
#include <QMatrix4x4>
#include <QOpenGLShaderProgram>
#include <QOpenGLBuffer>
#include <QOpenGLTexture>
#include <QSharedPointer>
#include <QTime>

#include "qvnc.h"

class VncRenderer : public QObject, public QQuickFramebufferObject::Renderer
{
    Q_OBJECT
public:
    VncRenderer(QQuickWindow *window,
                QSharedPointer<QOpenGLBuffer> vertices,
                QSharedPointer<QVector<GLuint>> copy_textures,
                QSharedPointer<QVector<QOpenGLTexture*>> draw_textures,
                QObject *parent = 0);
protected:
    void render() override;
    QOpenGLFramebufferObject *createFramebufferObject(const QSize &size) override;
    void synchronize(QQuickFramebufferObject *) override;
private:
    QSharedPointer<QOpenGLBuffer> m_vertices;
    QOpenGLShaderProgram m_program;
    QQuickWindow *m_window;
    QOpenGLFunctions *m_functions;
    bool m_clear;
    int m_frameCount;
    QTime m_time;

    unsigned int m_fill_count;
    unsigned int m_copy_count;
    unsigned int m_draw_count;

    QSharedPointer<QVector<GLuint>> m_copy_textures;
    QSharedPointer<QVector<QOpenGLTexture*>> m_draw_textures;
signals:
    void FBOTextureChanged(const unsigned int FBOTexture, const QMatrix4x4 &ortho);
    void finishedRendering();
};

#endif // VNCRENDERER_H
